--[[
Author: Lazy E Modding, Mantrid
https://www.lazyemodding.com
https://fs.mantrid.net/

Copyright: Ask Premission 
]]

local textureDir = g_currentModDirectory .. "textures/"

-- Load up some data
BaleTextureChanger = {
	modDir = g_currentModDirectory,
	customEnvironment = g_currentModName,
	baleSwaps = {
		{ 	
			isRoundbale = true, targetFillTypeName = "STRAW", targetWidth = 1.2, targetDiameter = 1.25, newBaleDiffuse = {
				BLUE_GOLD = textureDir .. "Straw/BlueGold/roundbaleStraw125_diffuse.dds",
				GREEN = textureDir .. "Straw/Green/roundbaleStraw125_diffuse.dds",
				RED_WHITE_BLUE = textureDir .. "Straw/RWB/roundbaleStraw125_diffuse.dds",
				WRAP_EDGE = textureDir .. "Straw/WrapEdge/roundbaleStraw125_diffuse.dds",
				DEFAULT = "data/objects/roundbales/roundbale125/roundbaleStraw125_diffuse.dds"
			}
		},
		{ 	
			isRoundbale = true, targetFillTypeName = "STRAW", targetWidth = 1.2, targetDiameter = 1.5, newBaleDiffuse = {
				BLUE_GOLD = textureDir .. "Straw/BlueGold/roundbaleStraw150_diffuse.dds",
				GREEN = textureDir .. "Straw/Green/roundbaleStraw150_diffuse.dds",
				RED_WHITE_BLUE = textureDir .. "Straw/RWB/roundbaleStraw150_diffuse.dds",
				WRAP_EDGE = textureDir .. "Straw/WrapEdge/roundbaleStraw150_diffuse.dds",
				DEFAULT = "data/objects/roundbales/roundbale150/roundbaleStraw150_diffuse.dds"
			}
		},
		{ 	
			isRoundbale = true, targetFillTypeName = "STRAW", targetWidth = 1.2, targetDiameter = 1.8, newBaleDiffuse = {
				BLUE_GOLD = textureDir .. "Straw/BlueGold/roundbaleStraw180_diffuse.dds",
				GREEN = textureDir .. "Straw/Green/roundbaleStraw180_diffuse.dds",
				RED_WHITE_BLUE = textureDir .. "Straw/RWB/roundbaleStraw180_diffuse.dds",
				WRAP_EDGE = textureDir .. "Straw/WrapEdge/roundbaleStraw180_diffuse.dds",
				DEFAULT = "data/objects/roundbales/roundbale180/roundbaleStraw180_diffuse.dds"
			}
		},
		{ 	
			isRoundbale = true, targetFillTypeName = "DRYGRASS_WINDROW", targetWidth = 1.2, targetDiameter = 1.25, newBaleDiffuse = {
				BLUE_GOLD = textureDir .. "Hay/BlueGold/roundbaleHay125_diffuse.dds",
				GREEN = textureDir .. "Hay/Green/roundbaleHay125_diffuse.dds",
				RED_WHITE_BLUE = textureDir .. "Hay/RWB/roundbaleHay125_diffuse.dds",
				WRAP_EDGE = textureDir .. "Hay/WrapEdge/roundbaleHay125_diffuse.dds",
				DEFAULT = "data/objects/roundbales/roundbale125/roundbaleHay125_diffuse.dds"
			}
		},
		{ 	
			isRoundbale = true, targetFillTypeName = "DRYGRASS_WINDROW", targetWidth = 1.2, targetDiameter = 1.5, newBaleDiffuse = {
				BLUE_GOLD = textureDir .. "Hay/BlueGold/roundbaleHay150_diffuse.dds",
				GREEN = textureDir .. "Hay/Green/roundbaleHay150_diffuse.dds",
				RED_WHITE_BLUE = textureDir .. "Hay/RWB/roundbaleHay150_diffuse.dds",
				WRAP_EDGE = textureDir .. "Hay/WrapEdge/roundbaleHay150_diffuse.dds",
				DEFAULT = "data/objects/roundbales/roundbale150/roundbaleHay150_diffuse.dds"
			}
		},
		{ 	
			isRoundbale = true, targetFillTypeName = "DRYGRASS_WINDROW", targetWidth = 1.2, targetDiameter = 1.8, newBaleDiffuse = {
				BLUE_GOLD = textureDir .. "Hay/BlueGold/roundbaleHay180_diffuse.dds",
				GREEN = textureDir .. "Hay/Green/roundbaleHay180_diffuse.dds",
				RED_WHITE_BLUE = textureDir .. "Hay/RWB/roundbaleHay180_diffuse.dds",
				WRAP_EDGE = textureDir .. "Hay/WrapEdge/roundbaleHay180_diffuse.dds",
				DEFAULT = "data/objects/roundbales/roundbale180/roundbaleHay180_diffuse.dds"
			}
		}
	},
	currentSettings = {
		STRAW125 = "DEFAULT",
		STRAW150 = "DEFAULT",
		STRAW180 = "DEFAULT",
		DRYGRASS_WINDROW125 = "DEFAULT",
		DRYGRASS_WINDROW150 = "DEFAULT",
		DRYGRASS_WINDROW180 = "DEFAULT"
	}
}


-- Triggers when new bales are created
function BaleTextureChanger:before_setFillType(fillTypeIndex, fillBale)
	local baleSwaps = BaleTextureChanger.baleSwaps
	for index = 1, #baleSwaps do
		if baleSwaps[index].isRoundbale == self.isRoundbale then
			
			-- Compare sizes and see if we have a match
			local isMatch = false
			if self.isRoundbale then
				isMatch = self.width == baleSwaps[index].targetWidth and self.diameter == baleSwaps[index].targetDiameter
			-- else
				-- isMatch = self.width == baleSwaps[index].targetWidth and self.height == baleSwaps[index].targetHeight and self.length == baleSwaps[index].targetLength
			end

			if isMatch then
				local targetFillTypeName = baleSwaps[index].targetFillTypeName
				local targetFillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(targetFillTypeName)
				if fillTypeIndex == targetFillTypeIndex then
					-- Found our target bale size, now get the right texture
					local baleKey = targetFillTypeName .. tostring(MathUtil.round(self.diameter, 2) * 100)
					local desiredSettingKey = BaleTextureChanger.currentSettings[baleKey]
					local newDiffuseFile = baleSwaps[index].newBaleDiffuse[desiredSettingKey]
					for i = 1, #self.fillTypes do
						if self.fillTypes[i].fillTypeIndex == fillTypeIndex then
							-- Apply texture
							self.fillTypes[i].diffuseFilename = newDiffuseFile
						end
					end
				end
			end

		end
	end
end

-- For already-out-in-the-world bales
function BaleTextureChanger.updateAllBales(updateList)
	local reference = {
		STRAW125 = { diameter=1.25, fillTypeIndex=36},
		STRAW150 = { diameter=1.5, fillTypeIndex=36},
		STRAW180 = { diameter=1.8, fillTypeIndex=36},
		DRYGRASS_WINDROW125 = { diameter=1.25, fillTypeIndex=35},
		DRYGRASS_WINDROW150 = { diameter=1.5, fillTypeIndex=35},
		DRYGRASS_WINDROW180 = { diameter=1.8, fillTypeIndex=35}
	}

	local itemsToSave = g_currentMission.itemSystem.itemsToSave
	local baleList = {}

	-- Loop through all the items
	for _, item in pairs(itemsToSave) do
		local object = item.item

		-- Check they're bales
		if object.isa ~= nil and object:isa(Bale) then
			-- Check they're on the update list
			if object.isRoundbale then -- Quickly dismiss non-round
				if updateList ~= nil then
					for key, _ in pairs(updateList) do
						if object.diameter == reference[key].diameter and object.fillType == reference[key].fillTypeIndex then
							table.insert(baleList, object)
						end
					end
				else
					-- Refresh all bales
					table.insert(baleList, object)
				end
			end
		end
	end

	-- Go through our bale list
	for i = #baleList, 1, -1 do
		for ii = 1, #baleList[i].fillTypes do
			local baleSelf = baleList[i]
			if baleSelf.fillTypes[ii] ~= nil and baleSelf.fillType ~= nil and baleSelf.fillTypes[ii].fillTypeIndex == baleSelf.fillType then
				BaleTextureChanger.before_setFillType(baleSelf, baleSelf.fillType) -- Note: We're passing in "self" manually, since the other function will otherwise use our mod environment, instead of the bale environment if called with : instead of .
				-- Now call this to update the texture
				baleSelf:setFillType(baleSelf.fillType, false)
			end
		end
	end

end

-- Called by the menu code, to pass changes for us to apply
function BaleTextureChanger.applyChangesFromMenu(changes)
	-- Record the changes as active settings
	if changes ~= nil then
		for key, value in pairs(changes) do
			BaleTextureChanger.currentSettings[key] = value
		end
	end
	
	-- Apply the changes
	BaleTextureChanger.updateAllBales(changes)
end

-- Our key binding calls this when pressed
function BaleTextureChanger:onInputOpenMenu(actionName, inputValue, callbackState, isAnalog, isMouse, deviceCategory, binding)
	g_gui:showGui("BaleTextureChangerMenu")
end

-- Registers our key binding
function BaleTextureChanger:registerActionEvents()
	local inputManager = g_currentMission.inputManager
	local _, eventId = inputManager:registerActionEvent(InputAction.BaleTextureChangerToggle, BaleTextureChanger, BaleTextureChanger.onInputOpenMenu, false, true, false, true)

	inputManager:setActionEventTextVisibility(eventId, true)
end

-- Save settings
function BaleTextureChanger:saveSettings()
	local self = BaleTextureChanger
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), g_currentMission.missionInfo.savegameIndex)
	end
	local key = "baleTextureChanger"
	local xmlFile = createXMLFile(key, savegameFolderPath .. "/mod_baleTextureChanger.xml", key)

	setXMLString(xmlFile, key .. ".STRAW125#value", self.currentSettings.STRAW125)
	setXMLString(xmlFile, key .. ".STRAW150#value", self.currentSettings.STRAW150)
	setXMLString(xmlFile, key .. ".STRAW180#value", self.currentSettings.STRAW180)
	setXMLString(xmlFile, key .. ".DRYGRASS_WINDROW125#value", self.currentSettings.DRYGRASS_WINDROW125)
	setXMLString(xmlFile, key .. ".DRYGRASS_WINDROW150#value", self.currentSettings.DRYGRASS_WINDROW150)
	setXMLString(xmlFile, key .. ".DRYGRASS_WINDROW180#value", self.currentSettings.DRYGRASS_WINDROW180)

	saveXMLFile(xmlFile)

	delete(xmlFile)
end

-- Load settings
function BaleTextureChanger:loadSettings()
	local self = BaleTextureChanger
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), g_currentMission.missionInfo.savegameIndex)
	end
	local key = "baleTextureChanger"

	if fileExists(savegameFolderPath .. "/mod_baleTextureChanger.xml") then
		
		local xmlfile = 0
		xmlFile = loadXMLFile(key, savegameFolderPath .. "/mod_baleTextureChanger.xml")

		self.currentSettings.STRAW125 = Utils.getNoNil(getXMLString(xmlFile, key .. ".STRAW125#value"), self.currentSettings.STRAW125)
		self.currentSettings.STRAW150 = Utils.getNoNil(getXMLString(xmlFile, key .. ".STRAW150#value"), self.currentSettings.STRAW150)
		self.currentSettings.STRAW180 = Utils.getNoNil(getXMLString(xmlFile, key .. ".STRAW180#value"), self.currentSettings.STRAW180)
		self.currentSettings.DRYGRASS_WINDROW125 = Utils.getNoNil(getXMLString(xmlFile, key .. ".DRYGRASS_WINDROW125#value"), self.currentSettings.DRYGRASS_WINDROW125)
		self.currentSettings.DRYGRASS_WINDROW150 = Utils.getNoNil(getXMLString(xmlFile, key .. ".DRYGRASS_WINDROW150#value"), self.currentSettings.DRYGRASS_WINDROW150)
		self.currentSettings.DRYGRASS_WINDROW180 = Utils.getNoNil(getXMLString(xmlFile, key .. ".DRYGRASS_WINDROW180#value"), self.currentSettings.DRYGRASS_WINDROW180)

		delete(xmlFile)
	end
	
	-- Now update all the bales!
	BaleTextureChanger.updateAllBales()
end
